// --8<--8<--8<--8<--
//
// Copyright (C) 2011 Smithsonian Astrophysical Observatory
//
// This file is part of bpmanip
//
// bpmanip is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// -->8-->8-->8-->8--

#include <cstdlib>
#include <vector>
#include <iostream>
#include <string>

using std::vector;
using std::cerr;
using std::endl;
using std::string;

#include <tracefctxx/TraceFct.h>

#include "BinaryPipe.h"
#include "Options.h"

int main ( int argc, char* argv [] ) {

    TraceFct( "bpmanip", 0, -1 );

    // Create a UserInput object, which reads in the
    //   arguments the user has given, and parses them.
    //
    Options options( argc, argv );

    // Create a BinaryPipe object associated with the input
    //   file given by the user (stored in the UserInput object).
    //
    BinaryPipe bin_pipe ( options.input, options.output );

    // AlterCore changes the "core" image (the image in memory) of the data
    //   packet.  This causes only the fields the user has asked for to
    //   be output.  AlterCore then freezes the core image so that it
    //   can't be changed for the duration of the program.
    //
    bin_pipe.AlterCore ( options.fields );

    bin_pipe.WriteHeader();

    bin_pipe.CopyPackets();

    return EXIT_SUCCESS;
}
