// --8<--8<--8<--8<--
//
// Copyright (C) 2011 Smithsonian Astrophysical Observatory
//
// This file is part of bpmanip
//
// bpmanip is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// -->8-->8-->8-->8--

#include <vector>
#include <string>

using std::string;
using std::vector;

#include "gtest/gtest.h"

#include "Wildmat.h"

TEST( Wildmat, OperatorExactEquals ) {

    Wildmat foo( "foo" );

    EXPECT_TRUE( foo == string("foo") );
    EXPECT_TRUE( string("foo") == foo );

    EXPECT_FALSE( foo == string("foo1") );
    EXPECT_FALSE( string("foo2") == foo );

}

TEST( Wildmat, OperatorPatternEquals ) {

    Wildmat pattern( "foo*" );

    EXPECT_TRUE( pattern == string("foo1") );
    EXPECT_TRUE( string("foo2") == pattern );

    EXPECT_FALSE( pattern == string("goo1") );
    EXPECT_FALSE( string("goo2") == pattern );

}

TEST( WildmatV, Operator ) {

    vector<Wildmat> patterns;
    patterns.push_back( string("foo??") );
    patterns.push_back( string("foo[0-9]") );
    patterns.push_back( string("goo*") );

    WildmatV check;

    EXPECT_TRUE( check( patterns, string("fooab") ) );
    EXPECT_TRUE( check( patterns, string("foo2") ) );
    EXPECT_TRUE( check( patterns, string("gooabc") ) );

    EXPECT_FALSE( check( patterns, string("fooa" ) ) );
    EXPECT_FALSE( check( patterns, string("foo0ac" ) ) );
    EXPECT_FALSE( check( patterns, string("go" ) ) );

}

