#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stddef.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include <bpipe/bpipe.h>
#include <bpipe/datatypes.h>
#include <bpipe/bpipe_axaf.h>

#include "main.h"
#include "common.h"
#include "process.h"

/* Routine to get the data pointer for a named data packet field */

static char *
dpktf_data( BPipe *bpipe, char *name, void *data )
{
  DpktField *field = bpipe_dpktf( bpipe, name );

  if ( field == NULL )
    tf_die( "%s isn't in input data stream", name);

  return bpipe_dpktf_data( field, data );
}

void
process(void)
{

  BPipe *bpipe;
  BPipeOutput *bpo;
  void  *data;
  size_t dpkt_core_size;
  double *weight;

  unsigned long n = 0;

  tf_enter("process");

  /* open up the photon streams */

  if ( NULL == ( bpipe = bpipe_new() ) )
    tf_die( "couldn't open bpipe: %s",
	    bpipe_strerror(bpipe_errno));

  if ( bpipe_input(bpipe, c.input) )
    tf_die( "couldn't open input file `%s': %s", c.input,
	    bpipe_strerror(bpipe_errno));

  if ( NULL == ( bpo = bpipe_output( bpipe, c.output ) ) )
    tf_die( "couldn't open input file `%s': %s", c.output,
	    bpipe_strerror(bpipe_errno));

  if ( NULL == (data = bpipe_map_alloc(bpipe,1, &dpkt_core_size)) )
    tf_exit( ExitERR_alloc, "unable to allocate memory for data packet: %s",
	    bpipe_strerror(bpipe_errno));

  weight   = (double *)   dpktf_data( bpipe, BP_AXAF_phot_wt, data);

  if ( bpipe_write_hdr(bpipe) )
    tf_die( "error writing output header: %s",
	    bpipe_strerror(bpipe_errno));

  /* read bpipe photon data packets */

  bpipe_errno = BPNOERROR;

  for( n = 1; 1 == bpipe_read_dpkts( bpipe, data, 1 ) ; n++ )
  {
      switch( c.op ) {

      case Add:      *weight += c.operand; break;
      case Subtract: *weight -= c.operand; break;
      case Multiply: *weight *= c.operand; break;
      case Divide:   *weight /= c.operand; break;
      case Equals:   *weight =  c.operand; break;

      default:
	  tf_die( "internal error: unknown operator: %d", c.op );
      }


    /* Write the data packet out  */
    if ( bpipe_write_dpkt( bpipe, data, bpo ) )
         tf_die( "error writing photon %d", n);

  }   /* End of main loop by ray data packets */

  if ( bpipe_errno != BPNOERROR )
    tf_die( "error in photon loop: %s",
	     bpipe_strerror( bpipe_errno ) );

  bpipe_delete(bpipe);
  tf_leave();
}
