local srcdir = os.getenv('srcdir') or '.'
local ea_db = table.concat( { srcdir, 'data', 'annulus.rdb'}, '/' )
local picket_db = table.concat( { srcdir, 'data', 'picket.rdb'}, '/' )

local raygen = require('saotrace.raygen')
local package = package

require('string')
require( 'deepcompare' )

local setfenv = setfenv

module( ..., package.seeall )

function setup()
   _G.setup()

   -- saotrace.config is reset using driver.setup(), so must be able
   -- to reload simple.lua to track changes to saotrace.config there
   package.loaded['chandra.saotrace.raygen.simple'] = nil
end

function test_point()

   local simple = require( 'chandra.saotrace.raygen.simple' )

   local env = getfenv(1 )
   setfenv(1, simple )

   point{ name = 'point',
	  spectrum = { 33, 44,
		       { type = 'picket', file = picket_db },
		    },
       }

   setfenv( 1, env )

   local stash = raygen.intrinsic.stash


   -- start the source
   assert_true( deepcompare( { { name = 'point'},
			       func = 'start',
			       mod = 'source' },
			     stash[1]),
		'start source' )

   -- the order of the following is dependent upon what happens in
   -- saotrace.raygen.source.create

   assert_equal( 'extent', stash[2].mod );
   assert_equal( 'point',    stash[2].func );

   assert_equal( 'spectrum', stash[3].mod );
   assert_equal( 'mono',     stash[3].func );
   assert_equal( 33,         stash[3][1].energy );
   assert_equal( 1,          stash[3][1].flux );

   assert_equal( 'spectrum', stash[4].mod );
   assert_equal( 'mono',     stash[4].func );
   assert_equal( 44,         stash[4][1].energy );
   assert_equal( 1,          stash[4][1].flux );

   assert_equal( 'spectrum', stash[5].mod );
   assert_equal( 'mono',     stash[5].func );
   assert_equal( 1,          stash[5][1].energy );
   assert_equal( 1,          stash[5][1].flux );

   assert_equal( 'spectrum', stash[6].mod );
   assert_equal( 'mono',     stash[6].func );
   assert_equal( 2,          stash[6][1].energy );
   assert_equal( 2,          stash[6][1].flux );

   -- finish the source
   assert_true( deepcompare( { { name = 'point'},
			       func = 'finish',
			       mod = 'source'
			    },
			     stash[#stash]), 'finish source' )



end


function test_ea()

   local config = require('saotrace.config').raygen
   config.scripts.entrance_aperture.args = { file = ea_db,
					     shell = 1,
					  }

   spectrum = 1.0
   raygen.init('chandra.saotrace.raygen.simple',
	       'spectrum = 1.49'
	    )

   local stash = raygen.intrinsic.stash[#raygen.intrinsic.stash]

   assert_equal( 'ea',       stash.mod );
   assert_equal( 'annulus',  stash.func );
   assert_equal( 10,         stash[1].opts.ro );

end


function test_jitter()

   local simple = require( 'chandra.saotrace.raygen.simple' )

   local env = getfenv(1 )
   setfenv(1, simple )

   dither_asol{ file = 'foo.data',
		ra = 0,
		dec = 90,
		roll = 180
	     }

   setfenv( 1, env )

   local stash = raygen.intrinsic.stash[1]

   local pi = require('math').pi
   assert_equal( 'jitter', stash.mod )
   assert_equal( 'file',   stash.func )
   assert_equal( pi, stash[1].optical_axis[1].angle[1] )

end
