require 'lunatest'

vobj = require( 'va' ).vobj

vspec = require( 'saotrace.raygen.ea').vspec

function test_annulus ()

   local spec = vspec.annulus

   local ok, args

   ok, args = vobj:validate( spec,
			'a1'
		     )

   assert_false( ok, args );
   assert_match( 'missing', args );

   ok, args = vobj:validate( spec,
			'a1',
			{  ri = -1, ro = 1 }
		     )

   assert_false( ok, args );
   assert_match( 'did not match type', args );

   ok, args = vobj:validate( spec,
			'a1',
			{  ri = 0.5,
			   ro = 1,
			}
		     )

   assert_true( ok, args );
   assert_equal( 1, args.opts.ro );
   assert_equal( 0.5, args.opts.ri );
   assert_equal( 0, args.opts.x );
   assert_equal( 0, args.opts.y );
   assert_equal( 0, args.opts.z );
   assert_equal( 'random', args.opts.raydist.type );

   ok, args = vobj:validate( spec,
			'a1',
			{  ri = 0.5,
			   ro = 1,
			   x = 3,
			   y = 4,
			   z = 5
			}
		     )

   assert_true( ok, args );
   assert_equal( 1, args.opts.ro );
   assert_equal( 0.5, args.opts.ri );
   assert_equal( 3, args.opts.x );
   assert_equal( 4, args.opts.y );
   assert_equal( 5, args.opts.z );

end

function test_circle ()

   local spec = vspec.circle

   local ok, args

   ok, args = vobj:validate( spec,
			'a1'
		     )

   assert_false( ok, args );
   assert_match( 'missing', args );

   ok, args = vobj:validate( spec,
			'a1',
			{  r = -1 }
		     )

   assert_false( ok, args );
   assert_match( 'did not match type', args );

   ok, args = vobj:validate( spec,
			'a1',
			{  r = 0 }
		     )

   assert_false( ok, args );
   assert_match( 'did not match type', args );

   ok, args = vobj:validate( spec,
			'a1',
			{  r = 1,
			}
		     )

   assert_true( ok, args );
   assert_equal( 1, args.opts.r );
   assert_equal( 0, args.opts.x );
   assert_equal( 0, args.opts.y );
   assert_equal( 0, args.opts.z );

   ok, args = vobj:validate( spec,
			'a1',
			{  r = 1,
			   x = 3,
			   y = 4,
			   z = 5
			}
		     )

   assert_true( ok, args );
   assert_equal( 1, args.opts.r );
   assert_equal( 3, args.opts.x );
   assert_equal( 4, args.opts.y );
   assert_equal( 5, args.opts.z );

end

function test_pinhole ()

   local ok, args

   local spec = vspec.pinhole

   ok, args = vobj:validate( spec, 'a1' )

   ok, args = vobj:validate( spec, 'a1', {  area = -1 })

   assert_false( ok, args );
   assert_match( 'did not match type', args );

   ok, args = vobj:validate( spec, 'a1', {  area = 0 })

   assert_false( ok, args );
   assert_match( 'did not match type', args );

   ok, args = vobj:validate( spec, 'a1', {  area = 2 } )

   assert_true( ok, args );
   assert_equal( 2, args.opts.area );
   assert_equal( 0, args.opts.x );
   assert_equal( 0, args.opts.y );
   assert_equal( 0, args.opts.z );

   ok, args = vobj:validate( spec, 'a1', 
			{  area = 0.5, x = 3, y = 4, z = 5})

   assert_true( ok, args );
   assert_equal( 0.5, args.opts.area );
   assert_equal( 3, args.opts.x );
   assert_equal( 4, args.opts.y );
   assert_equal( 5, args.opts.z );

end

lunatest.run()
