require 'lunatest'

va = require( 'saotrace.raygen.validate' )

units = require 'saotrace.raygen.units'

local math = require( 'math' )
local pi = math.pi
local deg2rad = pi / 180
local rad2deg = 180 / pi

function test_ra_good()

   for hms, comp in pairs {
      ['10:20:.5'] = { 10, 20, 0.5 },
      ['10:20:5'] = { 10, 20, 5 },
      ['0:2:5'] = { 0, 2, 5 },
   } do

      local degrees = 15 * ( comp[1] + ( comp[2] + comp[3] / 60 ) / 60 )
      local radians = degrees * deg2rad

      local ok, angle = va.validate_angle_ra( hms )

      assert_true( ok, angle )
      assert_equal( radians, angle[1], 1e-12 )

      local ok, angle = va.validate_angle_ra( degrees )
      assert_true( ok, angle )
      assert_equal( radians, angle[1], 1e-12 )

      local ok, angle = va.validate_angle_ra( { radians, 'radians' } )
      assert_true( ok, angle )
      assert_equal( radians, angle[1], 1e-12 )

   end

end

function test_ra_bad()

   for _,v in pairs{
      '10:20',
      '10:20:x',
      '10:20:.',
   } do

      local ok, angle = va.validate_angle_ra( v )
      assert_false( ok, angle )

   end

end


function test_dec_good()

   for hms, comp in pairs {
      ['-10:20:.5'] = { -1, 10, 20, 0.5 },
      ['+10:20:.5'] = {  1, 10, 20, 0.5 },
      ['10:20:.5']  = {  1, 10, 20, 0.5 },
   } do

      local degrees = comp[1] * ( comp[2] + ( comp[3] + comp[4] / 60 ) / 60 )
      local radians = degrees * deg2rad

      local ok, angle = va.validate_angle_dec( hms )

      assert_true( ok, angle )
      assert_equal( radians, angle[1], 1e-12 )

      local ok, angle = va.validate_angle_dec( degrees )
      assert_true( ok, angle )
      assert_equal( radians, angle[1], 1e-12 )

      local ok, angle = va.validate_angle_dec( { radians, 'radians' } )
      assert_true( ok, angle )
      assert_equal( radians, angle[1], 1e-12 )

   end

end

function test_dec_bad()

   for _,v in pairs{
      '10:20',
      '10:20:x',
      '10:20:.',
      '-90:1:1',
      '90:1:1',
   } do

      local ok, angle = va.validate_angle_dec( v )
      assert_false( ok, angle )

   end

end


function test_posquant()

   assert_true( va.posquant( { 3 } ) )
   assert_false( va.posquant( { -3 } ) )
   assert_false( va.posquant( { 0 } ) )

   assert_true( va.zposquant( { 3 } ) )
   assert_false( va.zposquant( { -3 } ) )
   assert_true( va.zposquant( { 0 } ) )

end
lunatest.run()
