/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of raygen
 *
 * raygen is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * raygen is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 *       Free Software Foundation, Inc.
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#include <lua.hpp>
#include <lualib.h>
#include <lauxlib.h>

#include <tracefctxx/TraceFct.h>
#include <tracefct/exiterrvals.h>

#include <suplib/str.h>

#include "main.h"
#include "common.h"

#include "lua_raygen.h"
#include "lua_system.h"
#include "misc.h"

#include "lua_interface.h"


/*---------------------------------------------------------------------*/

static char *
stack_top_msg( lua_State *L )
{
  const char *topmsg;
  char *msg;

  tf_assert( lua_gettop( L ) && (topmsg = lua_tostring(L, -1)) );
  tf_assert( msg = str_dup( topmsg ) );
  return msg;
}

static
int panic( lua_State *L ) {

    TraceFct::die( stack_top_msg( L ) );

    /* NOT REACHED (hopefully) */
    return 1;
}

lua_State *
init_lua( const char* cfgfile, const char* cfgpars )
{
    TraceFct tf("init_lua");
    lua_State *L;

    /* create interpreter instance */
    if ( NULL == ( L = luaL_newstate() ) )
        tf.die( "error creating Lua interpreter" );

    /* register a new panic function */
    lua_atpanic( L, panic );

    /* register standard lua libraries */
    luaL_openlibs( L );

    /* register raygen modules, both compiled and pure Lua */
    luaopen_raygen( L );
    luaopen_raygen_system( L );

    if ( cfgfile ) {

	if ( get_luaglobal( L, "saotrace.raygen", "init" ) )
	    tf.die( "internal error: couldn't find saotrace.raygen.init()" );

	lua_pushstring( L, cfgfile );
	lua_pushstring( L, cfgpars );
	lua_call( L, 2, 0 );
    }

    return L;
}


void
cleanup_lua( lua_State* L ) {

    lua_close( L );

}
