#include <string>
#include <sstream>
#include <iostream>

#include <lua.hpp>
#include <lauxlib.h>

#include "gtest/gtest.h"
#include "t_utils.h"

#include "lua_raygen.h"
#include "lua_fits.h"

using namespace std;

static int
panic_throw( lua_State *L ) {

    string errstr(  lua_tostring( L, -1 ) );
    lua_pop( L, -1 );
    throw errstr;
}

class StackTest : public testing::Test {

protected:

    lua_State *L;
    lua_CFunction panic;

    virtual void SetUp() {

	L = luaL_newstate();
	luaL_openlibs( L ) ;

	luaopen_raygen( L );

	panic = lua_atpanic( L, NULL );
	lua_atpanic( L, panic );

    }

    virtual void TearDown() {

	lua_close( L );

    }

    int dostring( string& str ) { return dostring( str.c_str() ); }
    int dostring( const char* str ) { return luaL_dostring( L, str ); }

};

TEST_F( StackTest, file ) {

    string test_file(getenv("RAYGEN_TEST_JITTER_TABLE"));
    ASSERT_NE( 0, test_file.length() );

    string cmd = "fits=require('saotrace.raygen.fits'); file = fits.file:new('" + test_file + "');";


    ASSERT_FALSE( dostring( cmd.c_str() ) ) << lua_tostring( L, -1 );

    ASSERT_FALSE( dostring( "return file:keyword_number( 'BITPIX' )" ))
	<< lua_tostring( L, -1 );

    ASSERT_EQ( 32, lua_tonumber( L, -1 ) );

    ASSERT_FALSE( dostring( "file:keyword( 'BITPIX' )" ))
	<< lua_tostring( L, -1 );

    ASSERT_EQ( string("32"), lua_tostring( L, -1 ) );
}

TEST_F( StackTest, table ) {

    string test_file(getenv("RAYGEN_TEST_JITTER_TABLE"));
    ASSERT_NE( 0, test_file.length() );

    string cmd = "fits=require('saotrace.raygen.fits'); file = fits.table:new('" + test_file + "');";

    ASSERT_FALSE( dostring( cmd.c_str() ) ) << lua_tostring( L, -1 );

    ASSERT_FALSE( dostring( "return file:keyword_number( 'NAXIS2' )" ))
	<< lua_tostring( L, -1 );

    ASSERT_EQ( 692, lua_tonumber( L, -1 ) );

    ASSERT_FALSE( dostring( "return file:keyword( 'RA_NOM' )" ))
	<< lua_tostring( L, -1 );

    ASSERT_EQ( string("3.3216395709286E+02"), lua_tostring( L, -1 ) );

    ASSERT_FALSE( dostring( "return file:exists_column('dy_err')" ) ) << lua_tostring( L, -1 );

    ASSERT_TRUE( lua_toboolean( L, -1 ) );

    ASSERT_FALSE( dostring( "return file:exists_column('nope, not here')" ) ) << lua_tostring( L, -1 );

    ASSERT_FALSE( lua_toboolean( L, -1 ) );

}

