module( ..., package.seeall )

local create = require('saotrace.baffles.ideal').create

local table = require( 'table' )
local os = require( 'os' )
local srcdir = os.getenv('srcdir') or '.'
local db = table.concat( { srcdir, 'data', 'geometry.rdb'}, '/' )

local giter = require('saotrace.geometry').read{ file = db, mode = 'iterator' }

local optic = {}

for geo in giter do

   if not optic[geo.optic] then
      optic[geo.optic] = {}
   end

   optic[geo.optic][geo.shell] = geo

end

setup = _G.setup

function test_outermost( )

   local inner, outer =
      create{ shell = 1,
	      optic = 'p',
	      cols = {
		 x = 'x0',
		 y = 'y0',
		 z = 'z_f',
		 r = 'rho_f',
	      },
	      db = db
       }

   -- test for correct radii of generated circles.
   -- first is p3
   assert_equal( optic.p[3].rho_f, inner.r, 'p3' )

   -- next is p1
   assert_equal( optic.p[1].rho_f, outer.r, 'p1' )
end

function test_middle( )

   local inner, outer =
   create{ shell = 3,
	  optic = 'p',
	  cols = {
	     x = 'x0',
	     y = 'y0',
	     z = 'z_f',
	     r = 'rho_f',
	  },
	  db = db
       }

   -- test for correct radii of generated circles.
   -- first is p4
   assert_equal( optic.p[4].rho_f, inner.r, 'p3' )

   -- next is p3

   assert_equal( optic.p[3].rho_f, outer.r, 'p1' )
end

function test_inner_disk( )

   local inner, outer =
      create{ shell = 6,
	  optic = 'p',
	  cols = {
	     x = 'x0',
	     y = 'y0',
	     z = 'z_f',
	     r = 'rho_f',
	  },
	  db = db,
	  inner_disk = { z = 20,
			 r = 10 },
       }

   -- test for correct radii of generated circles.
   -- first is p3
   assert_equal( 20, inner.z, 'p4-p6' )

   assert_equal( 10, inner.r, 'p4-p6' )

   -- next is p6

   assert_equal( optic.p[6].rho_f, outer.r, 'p6' )
end

function test_no_inner_disk( )

   local inner, outer =
   create{ shell = 6,
	  optic = 'p',
	  cols = {
	     x = 'x0',
	     y = 'y0',
	     z = 'z_f',
	     r = 'rho_f',
	  },
	  db = db
       }

   -- test for correct radii of generated circles.
   -- first is p3
   assert_equal( optic.p[6].rho_f - (optic.p[4].rho_f - optic.p[6].rho_f),
		 inner.r, 'p4-p6' )

   -- next is p6

   assert_equal( optic.p[6].rho_f, outer.r, 'p6' )
end
