module( ..., package.seeall )

local table = require( 'table' )
local os = require( 'os' )

local srcdir = os.getenv('srcdir') or '.'

local ea_db = table.concat( { srcdir, 'data', 'annulus.rdb'}, '/' )

local utils = require( 'saotrace.raygen.ea.utils.create_from_db' )
local va = require('validate.args')
local validate = va.validate

function setup ()
   _G.setup()
   saotrace.raygen.intrinsic.stash = {}
end


function test_1( )

   local nmatch = utils.create_from_db{ file = ea_db,
					match = { shell = 3 },
					type = 'annulus'
				     }

   assert_equal( 1, nmatch )
   local stash = saotrace.raygen.intrinsic.stash
   assert_equal( 10, stash[1][1].opts.ro )

end

function test_2( )

   local nmatch = utils.create_from_db{ file = ea_db,
					match = function( rec )
						   return rec.shell < 3
						end,
					type = 'annulus'
				     }

   assert_equal(2 , nmatch )
   local stash = saotrace.raygen.intrinsic.stash
   assert_equal( 10, stash[1][1].opts.ro )
   assert_equal(  8, stash[2][1].opts.ro )

end

function test_colmap_circle( )

   local nmatch = utils.create_from_db{ file = ea_db,
					match = function( rec )
						   return rec.shell < 3
						end,
					col = { r = 'ro' },
					type = 'circle'
				     }

   assert_equal(2 , nmatch )
   local stash = saotrace.raygen.intrinsic.stash
   assert_equal( 10, stash[1][1].opts.r )
   assert_equal(  8, stash[2][1].opts.r )

end

function test_colmap_point( )

   local nmatch = utils.create_from_db{ file = ea_db,
					match = function( rec )
						   return rec.shell < 3
						end,
					type = 'pinhole'
				     }

   assert_equal(2 , nmatch )
   local stash = saotrace.raygen.intrinsic.stash
   assert_equal( 0, stash[1][1].opts.x )
   assert_equal( 2, stash[2][1].opts.y )

end
