module( ..., package.seeall )

local vspec = require( 'saotrace.raygen.position.vspec' ):new()
local spec = { vspec:vspec() }

local vobj    = require('saotrace.raygen.validate').vobj:new()
local units   = require('saotrace.raygen.units')
local convert = units.convert

vobj:setopts{ named = false }

local coords = require ('saotrace.raygen.coords')
local min2rad = coords.min2rad
local deg2rad = coords.deg2rad

setup = _G.setup

function test_object ( )

   local nspec = vspec:new()

   assert_equal( 'raygen', nspec:default() )
   assert_equal( 'raygen', nspec:default('elaz') )
   assert_equal( 'elaz', nspec:default() )

   assert_error( function () nspec:default('non-existent') end )

   nspec:add( 'foobar', { vtable = { coord =  {enum = 'foobar' } } } )
   assert_equal( 'elaz', nspec:default( 'foobar' ) )
   assert_equal( 'foobar', nspec:default() )



end

function test_position_default ()

   local ok, val = vobj:validate( spec,
			     {
				theta = 3,
				phi = 2,
			     })
   assert_true( ok, val )

   assert_equal( 3, convert( val.theta, nil, units.angular.arcminute)[1] )
   assert_equal( 2, convert( val.phi,   nil, units.angular.degree)[1] )

end

function test_position_raygen ()

   local ok, val = vobj:validate( spec,
			     {
				theta = 7,
				phi = 44,
				coord = 'raygen'
			     })

   assert_true( ok, val )
   assert_equal(  7, convert( val.theta, nil, units.angular.arcminute)[1], 1e-6 )
   assert_equal( 44, convert( val.phi,   nil, units.angular.degree)[1] )


end

function test_position_osac_elaz ()

   local el = 100
   local az = 300

   local ok, val = vobj:validate( spec,
			     {
				el = el,
				az = az,
				coord = 'osac-elaz'
			     })

   el = el * min2rad
   az = az * min2rad

   local theta, phi = coords.osac_elaz_2_osac_polar( el, az )
   theta, phi = coords.osac_polar_2_raygen_polar( theta, phi  )


   assert_true( ok, val )
   assert_equal( theta, convert( val.theta, nil, units.angular.radian )[1], 1e-6 )
   assert_equal( phi,   convert( val.phi,   nil, units.angular.radian )[1] )

end

