/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of tparam
 *
 * tparam is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * tparam is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <tracefctxx/TraceFct.h>
#include <tracefct/exiterrvals.h>

#include "tparamxx.h"

paramfile
tparamopen(const char *filename, char **argv, int argc, const char *mode)
{
  paramfile pf;
  TraceFct tf("tparamopen");

  if (NULL == (pf = paramopen(filename, argv, argc, mode) ) )
  {
    if (filename)
      tf.die( "error opening parameter file `%s'", filename);
    else
      tf.die( "error opening parameter file" );
  }

  return pf;
}

#define TPGET(type,suffix) \
  type tpget##suffix(paramfile pf, const char *pname) \
  { \
    TraceFct tf("tpget" #suffix); \
    if (! paccess(pf, pname) ) \
    { \
      tf.die( "unable to access parameter %s", pname ); \
    } \
    return pget##suffix(pf, pname); \
  }

TPGET(short,s)
TPGET(int,i)
TPGET(double,d)
TPGET(float,f)
TPGET(int,b)

char *
tpgetstr(paramfile pf, const char *pname, char *val, int nchars,
	 int abort_if_empty)
{

  TraceFct tf("tpgetstr");

  /* check if parameter is really there */

  if (! paccess(pf, pname) )
    tf.die( "unable to access parameter %s", pname );

  pgetstr(pf, pname, val, nchars);

  if (abort_if_empty && val[0] == '\0')
    tf.die( "must specify %s", pname);

  return val;
}
